@@ -12,7 +12,7 @@ import corr, time, numpy, struct, sys, logging
 
 
 def exit_fail():
-    print 'FAILURE DETECTED. Log entries:\n',lh.printMessages()
+    print 'FAILURE DETECTED. Log entries:\n',c.log_handler.printMessages()
     print "Unexpected error:", sys.exc_info()
     try:
         c.disconnect_all()
@@ -46,21 +46,20 @@ if __name__ == '__main__':
         help='Print the ARP table.')
 
     opts, args = p.parse_args(sys.argv[1:])
-
-    if args==[]:
-        print 'Please specify a configuration file! \nExiting.'
-        exit()
-
     gbe_device='gbe'
 
-lh=corr.log_handlers.DebugLogHandler()
+    if args==[]:
+        config_file=None
+    else:
+        config_file=args[0]
+    verbose=opts.verbose
 
+try:        
+    print 'Connecting...',
+    c=corr.corr_functions.Correlator(config_file=config_file,log_level=logging.DEBUG if verbose else logging.INFO, connect=False)
+    c.connect()
+    print 'done'
 
-try:
-    print 'Loading the configuration file %s...'%args[0],
-    c=corr.corr_functions.Correlator(args[0],lh)
-    for logger in c.xloggers: logger.setLevel(10)
-    print 'done.'
     #assemble struct for header stuff...
     #0x00 - 0x07: My MAC address
     #0x08 - 0x0b: Not used
