@@ -7,25 +7,9 @@ Assumes the correlator is already initialsed and running etc.
 '''
 import corr, time, numpy, struct, sys, logging
 
-#brams
-brams=['bram_msb','bram_lsb','bram_oob']
-
-# OOB signalling bit offsets:
-ip_addr_bit_width = 32-8
-ip_addr_bit_offset = 6
-eof_bit = 5
-link_up_bit = 4
-tx_led_bit = 3
-tx_afull_bit = 2
-tx_over_bit = 1
-valid_bit = 0
-
-pkt_ip_mask = (2**(ip_addr_bit_width+ip_addr_bit_offset)) -(2**ip_addr_bit_offset)
-
 def exit_fail():
-    print 'FAILURE DETECTED. Log entries:\n',lh.printMessages()
+    print 'FAILURE DETECTED. Log entries:\n',c.log_handler.printMessages()
     print "Unexpected error:", sys.exc_info()
-
     try:
         c.disconnect_all()
     except: pass
@@ -39,16 +23,7 @@ def exit_clean():
     except: pass
     exit()
 
-def ip2str(pkt_ip):
-    ip_4 = (pkt_ip&((2**32)-(2**24)))>>24
-    ip_3 = (pkt_ip&((2**24)-(2**16)))>>16
-    ip_2 = (pkt_ip&((2**16)-(2**8)))>>8
-    ip_1 = (pkt_ip&((2**8)-(2**0)))>>0
-    #print 'IP:%i. decoded to: %i.%i.%i.%i'%(pkt_ip,ip_4,ip_3,ip_2,ip_1)
-    return '%i.%i.%i.%i'%(ip_4,ip_3,ip_2,ip_1)    
-
-def feng_unpack(f,hdr_index,pkt_len):
-    pkt_64bit = struct.unpack('>Q',bram_dmp['bram_msb'][f][(4*hdr_index):(4*hdr_index)+4]+bram_dmp['bram_lsb'][f][(4*hdr_index):(4*hdr_index)+4])[0]
+def feng_unpack(f, hdr_index, pkt_len):
     pkt_mcnt =(pkt_64bit&((2**64)-(2**16)))>>16
     pkt_ant = pkt_64bit&((2**16)-1)
     pkt_freq=pkt_mcnt%n_chans
@@ -58,18 +33,16 @@ def feng_unpack(f,hdr_index,pkt_len):
     sum_polI_r = 0
     sum_polI_i = 0
 
-    #average the packet contents - ignore first entry (header)
-    for pkt_index in range(1,(pkt_len)):
-        abs_index = hdr_index + pkt_index
-        pkt_64bit = struct.unpack('>Q',bram_dmp['bram_msb'][f][(4*abs_index):(4*abs_index)+4]+bram_dmp['bram_lsb'][f][(4*abs_index):(4*abs_index)+4])[0]
+    # average the packet contents - ignore first entry (header)
+    for pkt_index in range(1, pkt_len):
+        pkt_64bit = snap_data[f]['data'][pkt_index].data
 
         for offset in range(0,64,16):
             polQ_r = (pkt_64bit & ((2**(offset+16)) - (2**(offset+12))))>>(offset+12)
             polQ_i = (pkt_64bit & ((2**(offset+12)) - (2**(offset+8))))>>(offset+8)
             polI_r = (pkt_64bit & ((2**(offset+8)) - (2**(offset+4))))>>(offset+4)
             polI_i = (pkt_64bit & ((2**(offset+4)) - (2**(offset))))>>offset
-
-            #square each number and then sum it
+            # square each number and then sum it
             sum_polQ_r += (float(((numpy.int8(polQ_r << 4)>> 4)))/(2**binary_point))**2
             sum_polQ_i += (float(((numpy.int8(polQ_i << 4)>> 4)))/(2**binary_point))**2
             sum_polI_r += (float(((numpy.int8(polI_r << 4)>> 4)))/(2**binary_point))**2
@@ -129,33 +102,27 @@ if __name__ == '__main__':
         help='Be Verbose; print raw packet contents.')   
     p.add_option('-n', '--core_n', dest='core_n', type='int', default=0,
         help='Core number to decode. Default 0.')
-
-
     opts, args = p.parse_args(sys.argv[1:])
-
-    if args==[]:
-        print 'Please specify a configuration file! \nExiting.'
-        exit()
-
     if opts.man_trigger:
-        man_trig=True
+        man_trig = True
     else:
-        man_trig=False
+        man_trig = False
 
     if opts.raw:
-        man_valid=True
+        man_valid = True
     else:
-        man_valid=False
+        man_valid = False
+    if args == []:
+        config_file = None
+    else:
+        config_file = args[0]
+    verbose = opts.verbose
 
-    dev_name = 'snap_gbe_tx%i'%opts.core_n
-
-lh=corr.log_handlers.DebugLogHandler()
-
-try:
-    print 'Loading the configuration file %s...'%args[0],
-    c=corr.corr_functions.Correlator(args[0],lh)
-    for logger in c.xloggers: logger.setLevel(10)
-    print 'done.'
+try:        
+    print 'Connecting...',
+    c = corr.corr_functions.Correlator(config_file = config_file, log_level = logging.DEBUG if verbose else logging.INFO, connect = False)
+    c.connect()
+    print 'done'
 
     if opts.man_trigger:
         man_ctrl = (1<<1)+1
@@ -164,7 +131,9 @@ try:
     if opts.raw:
         man_ctrl += (1<<2)
 
-    report = dict()
+    print '------------------------'
+
+    dev_name = 'snap_gbe_tx%i' % opts.core_n
     binary_point = c.config['feng_fix_pnt_pos']
     num_bits = c.config['feng_bits']
     packet_len=c.config['10gbe_pkt_len']
@@ -172,70 +141,48 @@ try:
     n_chans = c.config['n_chans']
     n_ants_per_ibob=c.config['n_ants_per_xaui']
 
-    print '------------------------'
-    print 'Grabbing snap data...',
-
+    print 'Grabbing and unpacking snap data... ',
     if c.config['feng_out_type']=='xaui':
         servers = c.xsrvs
-        fpgas=c.xfpgas
-        bram_dmp=c.xsnap_all(dev_name,brams,man_trig=man_trig,man_valid=man_valid,wait_period=2)
+        fpgas = c.xfpgas
+        snap_data = corr.snap.get_gbe_tx_snapshot(c, dev_name, range(len(c.xfpgas)))
     else:
-        servers = c.fsrvs
-        fpgas=c.ffpgas
-        bram_dmp=c.fsnap_all(dev_name,brams,man_trig=man_trig,man_valid=man_valid,wait_period=2)
-    print 'done'
-
-#print 'BRAM DUMPS:'
-#print bram_dmp
-
-    print 'Unpacking bram contents...',
-    sys.stdout.flush()
-    bram_oob=dict()
-    for f,server in enumerate(servers):
-        if len(bram_dmp[brams[2]][f])<=4:
-            print '\n   No data for engine %s.'%server
-            bram_oob[f]={}
-        else:
-            if opts.verbose:
-                print '\n   Got %i values from %s.'%(len(bram_dmp[brams[2]][f])/4,server)
-            bram_oob[f]={'raw':struct.unpack('>%iL'%(len(bram_dmp[brams[2]][f])/4),bram_dmp[brams[2]][f])}
-            bram_oob[f].update({'eof':[bool(i & (2**eof_bit)) for i in bram_oob[f]['raw']]})
-            bram_oob[f].update({'valid':[bool(i & (2**valid_bit)) for i in bram_oob[f]['raw']]})
-            bram_oob[f].update({'link':[bool(i & (2**link_up_bit)) for i in bram_oob[f]['raw']]})
-            bram_oob[f].update({'tx_led':[bool(i & (2**tx_led_bit)) for i in bram_oob[f]['raw']]})
-            bram_oob[f].update({'tx_afull':[bool(i & (2**tx_afull_bit)) for i in bram_oob[f]['raw']]})
-            bram_oob[f].update({'tx_over':[bool(i & (2**tx_over_bit)) for i in bram_oob[f]['raw']]})
-            bram_oob[f].update({'ip_addr':[(i&pkt_ip_mask)>>ip_addr_bit_offset for i in bram_oob[f]['raw']]})
-            #print '\n\nFPGA %i, bramoob:'%f,bram_oob
-    print 'Done unpacking.'
+        raise RuntimeError('Not handled.')
+        #servers = c.fsrvs
+        #fpgas=c.ffpgas
+        #bram_dmp=c.fsnap_all(dev_name,brams,man_trig=man_trig,man_valid=man_valid,wait_period=2)
+    print 'done.'
 
+    report = dict()
     print 'Analysing packets:'
-    for f,fpga in enumerate(fpgas):
-        report[f]=dict()
-        report[f]['pkt_total']=0
+    for s in snap_data:
+        f = s['xfpga_index']
+        report[f] = dict()
+        report[f]['pkt_total'] = 0
         pkt_len = 0
-        prev_eof_index=-1
+        prev_eof_index = -1
+        report[f]['xfpga_index'] = f
 
-        for i in range(len(bram_dmp[brams[1]][f])/4):
+        for i in range(len(s['data'])):
             if opts.verbose or opts.raw:
-                pkt_64bit = struct.unpack('>Q',bram_dmp['bram_msb'][f][(4*i):(4*i)+4]+bram_dmp['bram_lsb'][f][(4*i):(4*i)+4])[0]
-                print '[%s] IDX: %i Contents: %016x'%(servers[f],i,pkt_64bit),
-                print '[%s]'%ip2str(bram_oob[f]['ip_addr'][i]),
-                if bram_oob[f]['valid'][i]: print '[valid]',
-                if bram_oob[f]['link'][i]: print '[link]',
-                if bram_oob[f]['tx_led'][i]: print '[tx_led]',
-                if bram_oob[f]['tx_afull'][i]: print '[TX buffer almost full!]',
-                if bram_oob[f]['tx_over'][i]: print '[TX buffer OVERFLOW!]',
-                if bram_oob[f]['eof'][i]: print '[EOF]',
+                pkt_64bit = s['data'][i].data
+                print '[%s] IDX: %i Contents: %016x' % (servers[f], i, pkt_64bit),
+                print '[%s]' % corr.corr_functions.ip2str(s['data'][i].ip_addr),
+                if s['data'][i].valid: print '[valid]',
+                if s['data'][i].link_up: print '[link up]',
+                if s['data'][i].led_tx: print '[led tx]',
+                if s['data'][i].tx_full: print '[TX buffer full]',
+                if s['data'][i].tx_over: print '[TX buffer OVERFLOW]',
+                if s['data'][i].eof: print '[EOF]',
                 print ''
 
-            if bram_oob[f]['eof'][i] and not opts.raw:
-                pkt_ip_str = ip2str(bram_oob[f]['ip_addr'][i])
-                print '[%s] EOF at %4i. Dest: %12s. Len: %3i. '%(servers[f],i,pkt_ip_str,i-prev_eof_index),
-                report[f]['pkt_total']+=1
-                hdr_index=prev_eof_index+1
-                pkt_len=i-prev_eof_index
-                prev_eof_index=i
+            if s['data'][i].eof and not opts.raw:
+                pkt_ip_str = corr.corr_functions.ip2str(s['data'][i].ip_addr)
+                print '[%s] EOF at %4i. Dest: %12s. Len: %3i. ' % (servers[f], i, pkt_ip_str, i - prev_eof_index),
+                report[f]['pkt_total'] += 1
+                hdr_index = prev_eof_index + 1
+                pkt_len = i - prev_eof_index
+                prev_eof_index = i
 
                 if not report[f].has_key('dest_ips'):
                     report[f].update({'dest_ips':{pkt_ip_str:1}})
@@ -244,16 +191,14 @@ try:
                 else:
                     report[f]['dest_ips'].update({pkt_ip_str:1})
 
-
-                if pkt_len != packet_len+1:
+                if pkt_len != packet_len + 1:
                     print 'Malformed Fengine Packet'
                     if not report[f].has_key('Malformed F-engine Packets'):
                         report[f]['Malformed F-engine Packets'] = 1
                     else:
                         report[f]['Malformed F-engine Packets'] += 1
-
                 else:
-                    feng_unpkd_pkt=feng_unpack(f,hdr_index,pkt_len)
+                    feng_unpkd_pkt = feng_unpack(f, hdr_index, pkt_len)
                     
                     ## Record the reception of the packet for this antenna, with this mcnt
                     #try: mcnts[f][feng_unpkd_pkt['pkt_mcnt']][feng_unpkd_pkt['pkt_ant']]=i
@@ -261,28 +206,25 @@ try:
                     #    mcnts[f][feng_unpkd_pkt['pkt_mcnt']]=numpy.zeros(n_ants,numpy.int)
                     #    mcnts[f][feng_unpkd_pkt['pkt_mcnt']][feng_unpkd_pkt['pkt_ant']]=i
                     #print mcnts
-                    print 'HDR @ %4i. MCNT %12u. Freq: %4i. Ant: %3i. 4 bit power: PolQ: %4.2f, PolI: %4.2f'%(hdr_index,feng_unpkd_pkt['pkt_mcnt'],feng_unpkd_pkt['pkt_freq'],feng_unpkd_pkt['pkt_ant'],feng_unpkd_pkt['rms_polQ'],feng_unpkd_pkt['rms_polI'])
+                    print 'HDR @ %4i. MCNT %12u. Freq: %4i. Ant: %3i. 4 bit power: PolQ: %4.2f, PolI: %4.2f' % (hdr_index, feng_unpkd_pkt['pkt_mcnt'], feng_unpkd_pkt['pkt_freq'], feng_unpkd_pkt['pkt_ant'], feng_unpkd_pkt['rms_polQ'], feng_unpkd_pkt['rms_polI'])
 
-                    if not report[f].has_key('Antenna%i'%feng_unpkd_pkt['pkt_ant']):
-                        report[f]['Antenna%i'%feng_unpkd_pkt['pkt_ant']] = 1
+                    if not report[f].has_key('Antenna%i' % feng_unpkd_pkt['pkt_ant']):
+                        report[f]['Antenna%i' % feng_unpkd_pkt['pkt_ant']] = 1
                     else:
-                        report[f]['Antenna%i'%feng_unpkd_pkt['pkt_ant']] += 1
-
-
-    print 'Done with all servers. \n REPORT:'
-
+                        report[f]['Antenna%i' % feng_unpkd_pkt['pkt_ant']] += 1
 
     print '\n\nDone with all servers.\nSummary:\n=========================='
-
-    for server,srvr in enumerate(servers):
+    for k, r in report.iteritems():
+        keys = report[k].keys()
+        keys.sort()
+        srvr = servers[r['xfpga_index']]
         print '------------------------'
         print srvr
         print '------------------------'
-        for key in report[server].iteritems():
-            print key[0],': ',key[1]
+        for key in keys:
+            print key,': ', r[key]
     print '=========================='
 
-
 except KeyboardInterrupt:
     exit_clean()
 except:
