@@ -6,14 +6,13 @@ Grabs the contents of "snap_descramble" for analysis. Valid for rev333 and onwar
 Author: Jason Manley
 
 Rev:
+2011-06-27  JRM Port to new snapshot blocks
 2010-07-29  JRM Port to corr-0.5.0
                 Added more useful summary logging.
 
 '''
 import corr, time, numpy, struct, sys, logging
 
-bram='bram'
-
 # OOB signalling bit offsets:
 data_bit_width =16
 data_bit_offset = 16
@@ -27,13 +26,15 @@ rcvd_bit = 0
 mcnt_mask = (2**(mcnt_bit_width+mcnt_bit_offset)) -(2**mcnt_bit_offset)
 data_mask = (2**(data_bit_width+data_bit_offset)) -(2**data_bit_offset)
 
+
 def exit_fail():
-    print 'FAILURE DETECTED. Log entries:\n',lh.printMessages()
+    print 'FAILURE DETECTED. Log entries:\n',c.log_handler.printMessages()
     print "Unexpected error:", sys.exc_info()
     try:
         c.disconnect_all()
     except: pass
-    raise
+    if verbose:
+        raise
     exit()
 
 def exit_clean():
@@ -42,6 +43,7 @@ def exit_clean():
     except: pass
     exit()
 
+
 def ip2str(pkt_ip):
     ip_4 = (pkt_ip&((2**32)-(2**24)))>>24
     ip_3 = (pkt_ip&((2**24)-(2**16)))>>16
@@ -138,10 +140,6 @@ if __name__ == '__main__':
 
     opts, args = p.parse_args(sys.argv[1:])
 
-    if args==[]:
-        print 'Please specify a configuration file! \nExiting.'
-        exit()
-
     if opts.man_trigger:
         man_trigger = True
         print 'NOTE: expected frequencies and antenna indices will be wrong with manual trigger option.'
@@ -159,13 +157,17 @@ if __name__ == '__main__':
     xeng_on_this_fpga = opts.core_n
     n_chans= opts.n_chans
 
-lh=corr.log_handlers.DebugLogHandler()
+    if args==[]:
+        config_file=None
+    else:
+        config_file=args[0]
+    verbose=opts.verbose
 
 try:
     print 'Connecting...',
-    c=corr.corr_functions.Correlator(args[0],lh)
-    for logger in c.xloggers: logger.setLevel(10)
-    print 'done.'
+    c=corr.corr_functions.Correlator(config_file=config_file,log_level=logging.DEBUG if verbose else logging.INFO,connect=False)
+    c.connect()
+    print 'done'
 
     servers = c.xsrvs
     binary_point = c.config['feng_fix_pnt_pos']
@@ -180,9 +182,9 @@ try:
     exp_len_from_descr = n_chans/n_xeng * n_ants * xeng_acc_len
 
     if opts.circ:
-        print 'Enabling circular-buffer capture on snap block.\n Triggering and Capturing, waiting 2 seconds for error...',
+        print 'Enabling circular-buffer capture on snap block.\n Triggering and Capturing, waiting for error...',
         sys.stdout.flush()
-        bram_dmp=c.xsnap_all(dev_name,[bram],man_trig=man_trigger,man_valid=raw_capture,wait_period=5,offset=0,circular_capture=True)
+        bram_dmp=corr.snap.snapshots_get(c.xfpgas,dev_name,man_trig=man_trigger,man_valid=raw_capture,wait_period=-1,offset=0,circular_capture=True)
         print 'done.'
     else:
         print 'Trying to retrieve %i words from each x engine...'%exp_len_from_descr
@@ -190,28 +192,28 @@ try:
         print '------------------------'
         #bram_dmp = [[0 for i in range(exp_len_from_descr*4)] for f in fpgas]
         print 'Triggering and capturing from offset 0 ...',
-        bram_dmp=c.xsnap_all(dev_name,[bram],man_trig=man_trigger,man_valid=raw_capture,wait_period=1) #,offset=0)
+        bram_dmp=corr.snap.snapshots_get(c.xfpgas,dev_name,man_trig=man_trigger,man_valid=raw_capture,wait_period=2,offset=0,circular_capture=False)
         print 'done'
         report = dict()
 
-        while bram_dmp['lengths'][0]<exp_len_from_descr:
+        while (bram_dmp['lengths'][0]/4)<exp_len_from_descr:
             print 'Triggering and capturing at offset %i...'%bram_dmp['lengths'][0],
-            bram_tmp=c.xsnap_all(dev_name,[bram],man_trig=man_trigger,man_valid=raw_capture,wait_period=1,offset=bram_dmp['lengths'][0])
+            bram_tmp=corr.snap.snapshots_get(c.xfpgas,dev_name,man_trig=man_trigger,man_valid=raw_capture,wait_period=1,offset=bram_dmp['lengths'][0],circular_capture=False)
             for f,fpga in enumerate(c.xfpgas):
-                bram_dmp[bram][f] += bram_tmp[bram][f]
+                bram_dmp['data'][f] += bram_tmp['data'][f]
             print 'done'
             for f,fpga in enumerate(c.xfpgas):
                 if (bram_tmp['lengths'][f] != bram_tmp['lengths'][f-1]): raise RuntimeError('Not all X engines captured the same amount of snapshot data.')
                 bram_dmp['lengths'][f] += bram_tmp['lengths'][f]
             time.sleep(0.1)
         for f,fpga in enumerate(c.xfpgas):
-            bram_dmp[bram][f]=''.join(bram_dmp[bram][f])
+            bram_dmp['data'][f]=''.join(bram_dmp['data'][f])
 
     #print 'BRAM DUMPS:'
     #print bram_dmp
 
     for f,fpga in enumerate(c.xfpgas):
-        print 'Got %i words starting at offset %i from %s'%(bram_dmp['lengths'][f],bram_dmp['offsets'][f],c.xsrvs[f])
+        print 'Got %i bytes starting at offset %i from %s'%(bram_dmp['lengths'][f],bram_dmp['offsets'][f],c.xsrvs[f])
 
 #print 'Total size for each x engine: %i bytes'%len(bram_dmp[0])
 
@@ -221,7 +223,7 @@ try:
     for f,fpga in enumerate(c.xfpgas):
         if bram_dmp['lengths'][f]==0: print 'Warning: got nothing back from snap block on %s.'%c.xsrvs[f]
         else:
-            bram_oob[f]={'raw':struct.unpack('>%iL'%(bram_dmp['lengths'][f]),bram_dmp[bram][f])}
+            bram_oob[f]={'raw':struct.unpack('>%iL'%(bram_dmp['lengths'][f]/4),bram_dmp['data'][f])}
             bram_oob[f].update({'rcvd':[bool(i & (2**rcvd_bit)) for i in bram_oob[f]['raw']]})
             bram_oob[f].update({'flag':[bool(i & (2**flag_bit)) for i in bram_oob[f]['raw']]})
             bram_oob[f].update({'valid':[bool(i & (2**valid_bit)) for i in bram_oob[f]['raw']]})
@@ -233,7 +235,7 @@ try:
     if opts.verbose:
         for f,server in enumerate(servers):
             i=bram_dmp['offsets'][f]
-            for ir in range(bram_dmp['lengths'][f]):
+            for ir in range(bram_dmp['lengths'][f]/4):
                 pkt_mcnt=bram_oob[f]['mcnt'][ir]
                 pkt_data=bram_oob[f]['data'][ir]
                 exp_ant=(i/xeng_acc_len)%n_ants
@@ -261,7 +263,7 @@ try:
         mcnts = dict()
         for f,server in enumerate(servers):
             report[f]=dict()
-            for i in range(0,bram_dmp['lengths'][f],xeng_acc_len):        
+            for i in range(0,bram_dmp['lengths'][f]/4,xeng_acc_len):        
                 pkt_mcnt=bram_oob[f]['mcnt'][i]
                 pkt_data=bram_oob[f]['data'][i]
                 exp_ant=(i/xeng_acc_len)%n_ants
