@@ -13,46 +13,51 @@ Revisions:
 """
 import corr, time, sys, struct, logging
 
+scroller = None
+screenData=[]
+
 def exit_fail():
-    scroller.screenTeardown()
     print 'FAILURE DETECTED. Log entries:\n',lh.printMessages()
     print "Unexpected error:", sys.exc_info()
     try:
         c.disconnect_all()
+        scroller.screenTeardown()
     except: pass
-    raise
-    
+    if verbose:
+        raise
     exit()
 
 def exit_clean():
-    scroller.screenTeardown()
     try:
         c.disconnect_all()
     except: pass
     exit()
 
+
 if __name__ == '__main__':
     from optparse import OptionParser
 
     p = OptionParser()
     p.set_usage('%prog [options] CONFIG_FILE')
     p.set_description(__doc__)
+    p.add_option('-v', '--verbose', dest='verbose',action='store_true', default=False,
+        help='Be verbose about errors.')
 
     opts, args = p.parse_args(sys.argv[1:])
 
     if args==[]:
-        print 'Please specify a configuration file! \nExiting.'
-        exit()
-
-lh=corr.log_handlers.DebugLogHandler()
-scroller = None
-screenData=[]
-
+        config_file=None
+    else:
+        config_file=args[0]
+    verbose=opts.verbose
+lh=corr.log_handlers.DebugLogHandler(35)
 try:
     print 'Connecting...',
-    c=corr.corr_functions.Correlator(args[0],lh)
-    for logger in c.xloggers: logger.setLevel(10)
-    print 'done.'
+    c=corr.corr_functions.Correlator(config_file=config_file,log_handler=lh,log_level=logging.DEBUG if verbose else logging.INFO,connect=False)
+    c.connect()
+    print 'done'
+
+
     # set up the curses scroll screen
     scroller = corr.scroll.Scroll()
     scroller.screenSetup()
