@@ -96,14 +96,19 @@ class FpgaClient(BlockingClient):
            """
         raise NotImplementedError("LISTCMD not implemented by client.")
 
-    def progdev(self, device_name):
-        """Return a status string for the given device.
+    def progdev(self, boffile):
+        """Program the FPGA with the specified boffile.
 
            @param self  This object.
-           @param device_name  String: name of the device.
+           @param boffile  String: name of the BOF file.
            @return  String: device status.
            """
-        reply, informs = self._request("progdev", device_name)
+        if boffile=='' or boffile==None:
+            reply, informs = self._request("progdev", '')
+            self._logger.info("Deprogramming FPGA... %s."%(reply.arguments[0]))
+        else:
+            reply, informs = self._request("progdev", boffile)
+            self._logger.info("Programming FPGA with %s... %s."%(boffile,reply.arguments[0]))
         return reply.arguments[0]
 
     def config_10gbe_core(self,device_name,mac,ip,port,arp_table,gateway=1):
@@ -546,7 +551,7 @@ class FpgaClient(BlockingClient):
          assert((type(qdr)==int))
          self.write_int('qdr%i_ctrl'%qdr,0xffffffff,blindwrite=True)
 
-    def get_snap(self, dev_name, brams, man_trig=False, man_valid=False, wait_period=1, offset=-1, circular_capture=False):
+    def get_snap(self, dev_name, brams, man_trig=False, man_valid=False, wait_period=1, offset=-1, circular_capture=False,word_mult=1):
         """Grabs all brams from a single snap block on this FPGA device.\n
             \tdev_name: string, name of the snap block.\n
             \tman_trig: boolean, Trigger the snap block manually.\n
@@ -554,10 +559,13 @@ class FpgaClient(BlockingClient):
             \tcircular_capture: boolean, Enable the circular capture function.\n
             \twait_period: integer, wait this number of seconds between triggering and trying to read-back the data. Make it negative to wait forever.\n
             \tbrams: list, names of the bram components.\n
+            \tword_mult: The snap block reports how many words were captured. Here we can specify how wide a word is (in 32-bit multiples) in order to retrieve the correct number of bytes. set to 2 for 64b snap blocks, 4 for 128b etc. Else we assume a 32-bit wide bram and pull addr*4 bytes.\n
             \tRETURNS: dictionary with keywords: \n
             \t\tlengths: list of integers matching number of valids captured off each fpga.\n
             \t\toffset: optional (depending on snap block version) list of number of valids elapsed since last  trigger on each fpga.
             \t\t{brams}: list of data from each fpga for corresponding bram.\n"""
+        #print "Deprecation warning: get_snap is to be deprecated. Please replace your design's %s block with a 'snapshot' block and use the snapshot_get function instead."%dev_name
+        self._logger.warn("Deprecation warning: get_snap is to be deprecated. Please replace your design's %s block with a 'snapshot' block and use the snapshot_get function instead."%dev_name)
         #2011-02-03 JRM added circular capture to trigger statement. 
         #               Invert logic for end detect. 
         #               Added wait forever option.
@@ -574,7 +582,7 @@ class FpgaClient(BlockingClient):
 
         done=False
         start_time=time.time()
-        while not (done and (offset!=0 or circular_capture)) and ((time.time()-start_time)<wait_period or (wait_period < 0)):
+        while not (done and (offset>=0 or circular_capture)) and ((time.time()-start_time)<wait_period or (wait_period < 0)):
             addr = self.read_uint(dev_name+'_addr')
             done = not bool(addr & 0x80000000)
 
@@ -584,7 +592,7 @@ class FpgaClient(BlockingClient):
         bram_dmp['offset']=0
         if (bram_size != self.read_uint(dev_name+'_addr')&0x7fffffff) or bram_size==0:
             #if address is still changing, then the snap block didn't finish capturing. we return empty.  
-            raise RuntimError("Looks like snap block didn't finish.")
+            raise RuntimeError("Looks like snap block didn't finish.")
             bram_dmp['length']=0
             bram_dmp['offset']=0
             bram_size=0
@@ -603,13 +611,106 @@ class FpgaClient(BlockingClient):
             if (bram_size == 0): 
                 bram_dmp[bram]=[]
             else: 
-                bram_dmp[bram]=(self.read(bram_path,(bram_size+1)*4))
+                bram_dmp[bram]=(self.read(bram_path,(bram_size+1)*4*word_mult))
         return bram_dmp
 
-        for b,bram in enumerate(brams):
-            bram_path = dev_name+'_'+bram
-            if (addr == 0): bram_dmp[bram]=[]
-            else: 
-                bram_dmp[bram]=(self.read(bram_path,(addr+1)*4))
+    def get_rcs(self,rcs_block_name='rcs'):
+        """Retrieves and decodes a revision control block."""
+        rv={}
+        rv['user']=self.read_uint(rcs_block_name+'_user')
+        app=self.read_uint(rcs_block_name+'_app')
+        lib=self.read_uint(rcs_block_name+'_lib')
+        if lib&(1<<31): 
+            rv['compile_timestamp']=lib&((2**31)-1)
+        else: 
+            if lib&(1<<30):
+                #type is svn
+                rv['lib_rcs_type']='svn'
+            else:
+                #type is git
+                rv['lib_rcs_type']='git'
+            if lib&(1<<28):
+                #dirty bit
+                rv['lib_dirty']=True
+            else:
+                rv['lib_dirty']=False
+            rv['lib_rev']=lib&((2**28)-1)
+        if app&(1<<31): 
+            rv['app_last_modified']=app&((2**31)-1)
+        else: 
+            if app&(1<<30):
+                #type is svn
+                rv['app_rcs_type']='svn'
+            else:
+                #type is git
+                rv['app_rcs_type']='git'
+            if app&(1<<28):
+                #dirty bit
+                rv['app_dirty']=True
+            else:
+                rv['lib_dirty']=False
+            rv['app_rev']=app&((2**28)-1)
+        return rv
+
+    def snapshot_get(self, dev_name, man_trig=False, man_valid=False, wait_period=1, offset=-1, circular_capture=False, get_extra_val=False):
+        """Grabs all brams from a single snap block on this FPGA device.\n
+            \tdev_name: string, name of the snap block.\n
+            \tman_trig: boolean, Trigger the snap block manually.\n
+            \toffset: integer, wait this number of bytes before beginning capture. Set to negative to ignore.\n
+            \tcircular_capture: boolean, Enable the circular capture function.\n
+            \twait_period: integer, wait this number of seconds between triggering and trying to read-back the data. Make it negative to wait forever.\n
+            \tRETURNS: dictionary with keywords: \n
+            \t\tlengths: number of bytes captured.\n
+            \t\toffset: number of bytes since last trigger.\n
+            \t\tdata: list of data from each fpga for corresponding bram.\n"""
+        # new snapshot block support (bytes instead of words) with hardware-configurable datawidth and user-selectable features.
+        #TODO Test offset, get_extra_val and circular capture modes.
+    
+        if offset >=0:
+            self.write_int(dev_name+'_trig_offset',offset)
+            #print 'Capturing from snap offset %i'%offset
+
+        #print 'Triggering Capture...',
+        self.write_int(dev_name+'_ctrl',(0 + (man_trig<<1) + (man_valid<<2) + (circular_capture<<3)))
+        self.write_int(dev_name+'_ctrl',(1 + (man_trig<<1) + (man_valid<<2) + (circular_capture<<3)))
+
+        done=False
+        start_time=time.time()
+        while not done and ((time.time()-start_time)<wait_period or (wait_period < 0)):
+            addr = self.read_uint(dev_name+'_status')
+            done = not bool(addr & 0x80000000)
+            time.sleep(0.05)
+
+        bram_size= addr&0x7fffffff
+        bram_dmp=dict()
+        bram_dmp['length']=bram_size
+        if (bram_size != self.read_uint(dev_name+'_status')&0x7fffffff) or bram_size==0:
+            #if address is still changing, then the snap block didn't finish capturing. we return empty.  
+            raise RuntimeError("A snap block logic error occurred or it didn't finish capturing in the allotted %2.2f seconds. Reported %i bytes captured."%(wait_period,bram_size))
+            bram_dmp['length']=0
+            bram_dmp['offset']=0
+            bram_size=0
+
+        if circular_capture:
+            #print 'offset: %i,tr_en_cnt: %i'%(offset,self.read_uint(dev_name+'_tr_en_cnt'))
+            # Snap block only starts incrementing tr_en_cnt after it has started writing into memory. Must thus add requested offset. Done later anyway.
+            bram_dmp['offset']=self.read_uint(dev_name+'_tr_en_cnt') - bram_size
+        else: 
+            bram_dmp['offset']=0
+
+        bram_dmp['offset']+=offset
+
+        if (bram_dmp['offset'] < 0):  
+            #you got a trigger and then a stop before the bram could even fill.
+            bram_dmp['offset']=0
+
+        if (bram_size == 0): 
+            bram_dmp['data']=[]
+        else: 
+            bram_dmp['data']=(self.read(dev_name+'_bram',(bram_size)))
+
+        if get_extra_val==True:
+            bram_dmp['val']=self.read_uint(dev_name+'_val')
+
         return bram_dmp
 
